@@ -96,14 +96,22 @@ class Page(object):
             Callable to convert an item from the type in the raw API response
             into the native object. Will be called with the iterator and a
             single item.
+        raw_page Optional[google.protobuf.message.Message]:
+            The raw page response.
     """
 
-    def __init__(self, parent, items, item_to_value):
+    def __init__(self, parent, items, item_to_value, raw_page=None):
         self._parent = parent
         self._num_items = len(items)
         self._remaining = self._num_items
         self._item_iter = iter(items)
         self._item_to_value = item_to_value
+        self._raw_page = raw_page
+
+    @property
+    def raw_page(self):
+        """google.protobuf.message.Message"""
+        return self._raw_page
 
     @property
     def num_items(self):
@@ -154,8 +162,13 @@ class Iterator(object):
         max_results (int): The maximum number of results to fetch.
     """
 
-    def __init__(self, client, item_to_value=_item_to_value_identity,
-                 page_token=None, max_results=None):
+    def __init__(
+        self,
+        client,
+        item_to_value=_item_to_value_identity,
+        page_token=None,
+        max_results=None,
+    ):
         self._started = False
         self.client = client
         """Optional[Any]: The client that created this iterator."""
@@ -166,7 +179,7 @@ class Iterator(object):
             single item.
         """
         self.max_results = max_results
-        """int: The maximum number of results to fetch."""
+        """int: The maximum number of results to fetch"""
 
         # The attributes below will change over the life of the iterator.
         self.page_number = 0
@@ -190,7 +203,7 @@ class Iterator(object):
             ValueError: If the iterator has already been started.
         """
         if self._started:
-            raise ValueError('Iterator has already started', self)
+            raise ValueError("Iterator has already started", self)
         self._started = True
         return self._page_iter(increment=True)
 
@@ -211,7 +224,7 @@ class Iterator(object):
             ValueError: If the iterator has already been started.
         """
         if self._started:
-            raise ValueError('Iterator has already started', self)
+            raise ValueError("Iterator has already started", self)
         self._started = True
         return self._items_iter()
 
@@ -285,7 +298,8 @@ class HTTPIterator(Iterator):
             can be found.
         page_token (str): A token identifying a page in a result set to start
             fetching results from.
-        max_results (int): The maximum number of results to fetch.
+        page_size (int): The maximum number of results to fetch per page
+        max_results (int): The maximum number of results to fetch
         extra_params (dict): Extra query string parameters for the
             API call.
         page_start (Callable[
@@ -301,24 +315,35 @@ class HTTPIterator(Iterator):
     .. autoattribute:: pages
     """
 
-    _DEFAULT_ITEMS_KEY = 'items'
-    _PAGE_TOKEN = 'pageToken'
-    _MAX_RESULTS = 'maxResults'
-    _NEXT_TOKEN = 'nextPageToken'
+    _DEFAULT_ITEMS_KEY = "items"
+    _PAGE_TOKEN = "pageToken"
+    _MAX_RESULTS = "maxResults"
+    _NEXT_TOKEN = "nextPageToken"
     _RESERVED_PARAMS = frozenset([_PAGE_TOKEN])
-    _HTTP_METHOD = 'GET'
+    _HTTP_METHOD = "GET"
 
-    def __init__(self, client, api_request, path, item_to_value,
-                 items_key=_DEFAULT_ITEMS_KEY,
-                 page_token=None, max_results=None, extra_params=None,
-                 page_start=_do_nothing_page_start, next_token=_NEXT_TOKEN):
+    def __init__(
+        self,
+        client,
+        api_request,
+        path,
+        item_to_value,
+        items_key=_DEFAULT_ITEMS_KEY,
+        page_token=None,
+        page_size=None,
+        max_results=None,
+        extra_params=None,
+        page_start=_do_nothing_page_start,
+        next_token=_NEXT_TOKEN,
+    ):
         super(HTTPIterator, self).__init__(
-            client, item_to_value, page_token=page_token,
-            max_results=max_results)
+            client, item_to_value, page_token=page_token, max_results=max_results
+        )
         self.api_request = api_request
         self.path = path
         self._items_key = items_key
         self.extra_params = extra_params
+        self._page_size = page_size
         self._page_start = page_start
         self._next_token = next_token
         # Verify inputs / provide defaults.
@@ -332,11 +357,9 @@ class HTTPIterator(Iterator):
         Raises:
             ValueError: If a reserved parameter is used.
         """
-        reserved_in_use = self._RESERVED_PARAMS.intersection(
-            self.extra_params)
+        reserved_in_use = self._RESERVED_PARAMS.intersection(self.extra_params)
         if reserved_in_use:
-            raise ValueError('Using a reserved parameter',
-                             reserved_in_use)
+            raise ValueError("Using a reserved parameter", reserved_in_use)
 
     def _next_page(self):
         """Get the next page in the iterator.
@@ -348,7 +371,7 @@ class HTTPIterator(Iterator):
         if self._has_next_page():
             response = self._get_next_page_response()
             items = response.get(self._items_key, ())
-            page = Page(self, items, self.item_to_value)
+            page = Page(self, items, self.item_to_value, raw_page=response)
             self._page_start(self, page, response)
             self.next_page_token = response.get(self._next_token)
             return page
@@ -379,8 +402,18 @@ class HTTPIterator(Iterator):
         result = {}
         if self.next_page_token is not None:
             result[self._PAGE_TOKEN] = self.next_page_token
+
+        page_size = None
         if self.max_results is not None:
-            result[self._MAX_RESULTS] = self.max_results - self.num_results
+            page_size = self.max_results - self.num_results
+            if self._page_size is not None:
+                page_size = min(page_size, self._page_size)
+        elif self._page_size is not None:
+            page_size = self._page_size
+
+        if page_size is not None:
+            result[self._MAX_RESULTS] = page_size
+
         result.update(self.extra_params)
         return result
 
@@ -394,18 +427,16 @@ class HTTPIterator(Iterator):
             ValueError: If the HTTP method is not ``GET`` or ``POST``.
         """
         params = self._get_query_params()
-        if self._HTTP_METHOD == 'GET':
+        if self._HTTP_METHOD == "GET":
             return self.api_request(
-                method=self._HTTP_METHOD,
-                path=self.path,
-                query_params=params)
-        elif self._HTTP_METHOD == 'POST':
+                method=self._HTTP_METHOD, path=self.path, query_params=params
+            )
+        elif self._HTTP_METHOD == "POST":
             return self.api_request(
-                method=self._HTTP_METHOD,
-                path=self.path,
-                data=params)
+                method=self._HTTP_METHOD, path=self.path, data=params
+            )
         else:
-            raise ValueError('Unexpected HTTP method', self._HTTP_METHOD)
+            raise ValueError("Unexpected HTTP method", self._HTTP_METHOD)
 
 
 class _GAXIterator(Iterator):
@@ -425,8 +456,11 @@ class _GAXIterator(Iterator):
 
     def __init__(self, client, page_iter, item_to_value, max_results=None):
         super(_GAXIterator, self).__init__(
-            client, item_to_value, page_token=page_iter.page_token,
-            max_results=max_results)
+            client,
+            item_to_value,
+            page_token=page_iter.page_token,
+            max_results=max_results,
+        )
         self._gax_page_iter = page_iter
 
     def _next_page(self):
@@ -474,21 +508,23 @@ class GRPCIterator(Iterator):
     .. autoattribute:: pages
     """
 
-    _DEFAULT_REQUEST_TOKEN_FIELD = 'page_token'
-    _DEFAULT_RESPONSE_TOKEN_FIELD = 'next_page_token'
+    _DEFAULT_REQUEST_TOKEN_FIELD = "page_token"
+    _DEFAULT_RESPONSE_TOKEN_FIELD = "next_page_token"
 
     def __init__(
-            self,
-            client,
-            method,
-            request,
-            items_field,
-            item_to_value=_item_to_value_identity,
-            request_token_field=_DEFAULT_REQUEST_TOKEN_FIELD,
-            response_token_field=_DEFAULT_RESPONSE_TOKEN_FIELD,
-            max_results=None):
+        self,
+        client,
+        method,
+        request,
+        items_field,
+        item_to_value=_item_to_value_identity,
+        request_token_field=_DEFAULT_REQUEST_TOKEN_FIELD,
+        response_token_field=_DEFAULT_RESPONSE_TOKEN_FIELD,
+        max_results=None,
+    ):
         super(GRPCIterator, self).__init__(
-            client, item_to_value, max_results=max_results)
+            client, item_to_value, max_results=max_results
+        )
         self._method = method
         self._request = request
         self._items_field = items_field
@@ -506,14 +542,13 @@ class GRPCIterator(Iterator):
             return None
 
         if self.next_page_token is not None:
-            setattr(
-                self._request, self._request_token_field, self.next_page_token)
+            setattr(self._request, self._request_token_field, self.next_page_token)
 
         response = self._method(self._request)
 
         self.next_page_token = getattr(response, self._response_token_field)
         items = getattr(response, self._items_field)
-        page = Page(self, items, self.item_to_value)
+        page = Page(self, items, self.item_to_value, raw_page=response)
 
         return page
 
